@@ -1,5 +1,5 @@
 /* gnome-href.c
- * Copyright (C) 1998, James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998-2001, James Henstridge <james@daa.com.au>
  * All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -31,14 +31,14 @@
 #include <gtk/gtk.h>
 #include <libgnome/gnome-url.h>
 #include "gnome-href.h"
+#include "libgnomeui-access.h"
+#include "gnome-url.h"
 
 struct _GnomeHRefPrivate {
 	gchar *url;
 	GtkWidget *label;
 };
 
-static void gnome_href_class_init	(GnomeHRefClass *klass);
-static void gnome_href_instance_init	(GnomeHRef *href);
 static void gnome_href_clicked		(GtkButton *button);
 static void gnome_href_destroy		(GtkObject *object);
 static void gnome_href_finalize		(GObject *object);
@@ -51,6 +51,8 @@ static void gnome_href_set_property	(GObject *object,
 					 const GValue * value,
 					 GParamSpec * pspec);
 static void gnome_href_realize		(GtkWidget *widget);
+static void gnome_href_style_set        (GtkWidget *widget,
+					 GtkStyle *previous_style);
 static void drag_data_get    		(GnomeHRef          *href,
 					 GdkDragContext     *context,
 					 GtkSelectionData   *selection_data,
@@ -73,11 +75,11 @@ static const GtkTargetEntry other_drop_types[] = {
 	{ "_NETSCAPE_URL",       0, 0 }
 };
 
-static const gint n_http_drop_types = 
+static const gint n_http_drop_types =
    sizeof(http_drop_types) / sizeof(http_drop_types[0]);
-static const gint n_ftp_drop_types = 
+static const gint n_ftp_drop_types =
    sizeof(ftp_drop_types) / sizeof(ftp_drop_types[0]);
-static const gint n_other_drop_types = 
+static const gint n_other_drop_types =
    sizeof(other_drop_types) / sizeof(other_drop_types[0]);
 
 
@@ -94,7 +96,7 @@ enum {
  **/
 /* The following defines the get_type */
 GNOME_CLASS_BOILERPLATE (GnomeHRef, gnome_href,
-			 GtkButton, gtk_button, GTK_TYPE_BUTTON)
+			 GtkButton, GTK_TYPE_BUTTON)
 
 static void
 gnome_href_class_init (GnomeHRefClass *klass)
@@ -103,14 +105,15 @@ gnome_href_class_init (GnomeHRefClass *klass)
 	GObjectClass *gobject_class = (GObjectClass *)klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
 	GtkButtonClass *button_class = (GtkButtonClass *)klass;
-	
+
 	object_class->destroy = gnome_href_destroy;
-	
+
 	gobject_class->finalize = gnome_href_finalize;
 	gobject_class->set_property = gnome_href_set_property;
 	gobject_class->get_property = gnome_href_get_property;
 
 	widget_class->realize = gnome_href_realize;
+	widget_class->style_set = gnome_href_style_set;
 	button_class->clicked = gnome_href_clicked;
 
 	/* By default we link to The World Food Programme */
@@ -131,6 +134,12 @@ gnome_href_class_init (GnomeHRefClass *klass)
 							      (G_PARAM_READABLE |
 							       G_PARAM_WRITABLE)));
 
+	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (gobject_class),
+						 g_param_spec_boxed ("link_color",
+								     _("Link color"),
+								     _("Color used to draw the link"),
+								     GDK_TYPE_COLOR,
+								     G_PARAM_READABLE));
 }
 
 static void
@@ -141,15 +150,25 @@ gnome_href_instance_init (GnomeHRef *href)
 	href->_priv->label = gtk_label_new("");
 	gtk_widget_ref(href->_priv->label);
 
+
 	gtk_button_set_relief(GTK_BUTTON(href), GTK_RELIEF_NONE);
+
 	gtk_container_add(GTK_CONTAINER(href), href->_priv->label);
 	gtk_widget_show(href->_priv->label);
 
 	href->_priv->url = NULL;
 
 	/* the source dest is set on set_url */
-	gtk_signal_connect (GTK_OBJECT (href), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get), NULL);
+	g_signal_connect (href, "drag_data_get",
+			  G_CALLBACK (drag_data_get), NULL);
+
+	/* Set our accessible description.  We don't set the name as we want it
+	 * to be the contents of the label, which is the default anyways.
+	 */
+
+	_add_atk_name_desc (GTK_WIDGET (href),
+			    NULL,
+			    _("This button will take you to the URI that it displays."));
 }
 
 static void
@@ -279,14 +298,14 @@ void gnome_href_set_url(GnomeHRef *href, const gchar *url) {
  * Returns:  Pointer to text contained in the label widget.
  **/
 
-const gchar *gnome_href_get_text(GnomeHRef *href) {
-  gchar *ret;
+const gchar *
+gnome_href_get_text(GnomeHRef *href)
+{
 
   g_return_val_if_fail(href != NULL, NULL);
   g_return_val_if_fail(GNOME_IS_HREF(href), NULL);
 
-  gtk_label_get(GTK_LABEL(href->_priv->label), &ret);
-  return ret;
+  return gtk_label_get_text (GTK_LABEL(href->_priv->label));
 }
 
 
@@ -302,17 +321,16 @@ const gchar *gnome_href_get_text(GnomeHRef *href) {
 void
 gnome_href_set_text (GnomeHRef *href, const gchar *text)
 {
-  gchar *pattern;
+  gchar *markup;
 
   g_return_if_fail(href != NULL);
   g_return_if_fail(GNOME_IS_HREF(href));
   g_return_if_fail(text != NULL);
 
-  /* pattern used to set underline for string */
-  pattern = g_strnfill(strlen(text), '_');
-  gtk_label_set_text(GTK_LABEL(href->_priv->label), text);
-  gtk_label_set_pattern(GTK_LABEL(href->_priv->label), pattern);
-  g_free(pattern);
+  /* underline the string */
+  markup = g_strdup_printf("<u>%s</u>", text);
+  gtk_label_set_markup(GTK_LABEL(href->_priv->label), markup);
+  g_free(markup);
 }
 
 #ifndef GNOME_DISABLE_DEPRECATED_SOURCE
@@ -321,8 +339,9 @@ gnome_href_set_text (GnomeHRef *href, const gchar *text)
  * gnome_href_get_label
  * @href: Pointer to GnomeHRef widget
  *
- * Description:
- * deprecated, use #gnome_href_get_text
+ * Deprecated, use #gnome_href_get_text.
+ *
+ * Returns:
  **/
 
 const gchar *
@@ -362,62 +381,63 @@ gnome_href_clicked (GtkButton *button)
   g_return_if_fail(button != NULL);
   g_return_if_fail(GNOME_IS_HREF(button));
 
-  GNOME_CALL_PARENT_HANDLER (GTK_BUTTON_CLASS, clicked, (button));
+  GNOME_CALL_PARENT (GTK_BUTTON_CLASS, clicked, (button));
 
   href = GNOME_HREF(button);
 
   g_return_if_fail(href->_priv->url != NULL);
 
-  /* FIXME: Use the error variable from gnome_url_show */
-  if(!gnome_url_show(href->_priv->url, NULL)) {
-      GtkWidget *dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("Error occured while trying to launch the "
-						   "URL handler.\n"
-						   "Please check the settings in the "
-						   "Control Center if they are correct."));
-
-      gtk_dialog_run(GTK_DIALOG(dialog));
+  /* FIXME: Use the error variable from gnome_url_show_on_screen */
+  if(!gnome_url_show_on_screen (href->_priv->url, gtk_widget_get_screen (GTK_WIDGET (href)), NULL)) {
+    GtkWidget *dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("An error has occured while trying to launch the "
+						 "default web browser.\n"
+						 "Please check your settings in the "
+						 "'Preferred Applications' preference tool."));
+    
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
   }
 }
 
 static void
-gnome_href_destroy(GtkObject *object)
+gnome_href_destroy (GtkObject *object)
 {
 	GnomeHRef *href;
 
 	/* remember, destroy can be run multiple times! */
 
-	g_return_if_fail(object != NULL);
-	g_return_if_fail(GNOME_IS_HREF(object));
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GNOME_IS_HREF(object));
 
-	href = GNOME_HREF(object);
+	href = GNOME_HREF (object);
 
-	g_free(href->_priv->url);
-	href->_priv->url = NULL;
-
-	if(href->_priv->label) {
-		gtk_widget_unref(href->_priv->label);
+	if (href->_priv->label != NULL) {
+		gtk_widget_unref (href->_priv->label);
 		href->_priv->label = NULL;
 	}
 
-	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
-gnome_href_finalize(GObject *object)
+gnome_href_finalize (GObject *object)
 {
 	GnomeHRef *href;
 
-	g_return_if_fail(object != NULL);
-	g_return_if_fail(GNOME_IS_HREF(object));
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GNOME_IS_HREF(object));
 
-	href = GNOME_HREF(object);
+	href = GNOME_HREF (object);
 
-	g_free(href->_priv);
+	g_free (href->_priv->url);
+	href->_priv->url = NULL;
+
+	g_free (href->_priv);
 	href->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
@@ -425,11 +445,41 @@ gnome_href_realize(GtkWidget *widget)
 {
 	GdkCursor *cursor;
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, realize, (widget));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, realize, (widget));
 
-	cursor = gdk_cursor_new(GDK_HAND2);
-	gdk_window_set_cursor(widget->window, cursor);
-	gdk_cursor_destroy(cursor);
+	cursor = gdk_cursor_new (GDK_HAND2);
+	gdk_window_set_cursor (GTK_BUTTON (widget)->event_window, cursor);
+	gdk_cursor_unref (cursor);
+}
+
+static void
+gnome_href_style_set (GtkWidget *widget,
+		      GtkStyle *previous_style)
+{
+	GdkColor *link_color;
+	GdkColor blue = { 0, 0x0000, 0x0000, 0xffff };
+	GnomeHRef *href;
+
+	href = GNOME_HREF (widget);
+	
+	gtk_widget_style_get (widget,
+			      "link_color", &link_color,
+			      NULL);
+
+	if (!link_color)
+		link_color = &blue;
+
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_NORMAL, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_ACTIVE, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_PRELIGHT, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_SELECTED, link_color);
+
+	if (link_color != &blue)
+		gdk_color_free (link_color);
 }
 
 static void
